--[[ Colors:
	cffBF5FFF
	cffFF69B4
]]

CookyChat = LibStub("AceAddon-3.0"):NewAddon("CookyChat","AceConsole-3.0")

-- Create frame to handle on script triggering
CookyChatHandler = CreateFrame("Frame")

-- Handler for creating GUI elements
local AceGUI = LibStub("AceGUI-3.0")

-- Variable to control opening the gui
local frameOpen

-- Different tooltips for reply based on trigger type selected
local achivementReplyTooltip = "The message that will be sent. Use %S to inset the name of the person who earned the achivement, use %A to link the achivement"
local chatReplyTooltip = 'The message that will be sent. Use %S to insert the name of the person talking, and %F for the "NAME-REALM" version of the name of the person talking.'
local guildReplyTooltip = "The message that will be sent. Use %S to insert the name of the person joining, and %G for the name of the guild."

-- Current Version
local CURRENT_VERSION = 10

-- Handler and db for minimap icon
local icon = LibStub("LibDBIcon-1.0", true)
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local CookyChatLDB = ldb:NewDataObject("CookyChat", {
	label = "CookyChat",
	type = "data source",
	text = "CookyChat",
	icon = "Interface\\AddOns\\CookyChat\\Media\\icon",
})

-- Refreshes minimap button
function CookyChat:RefreshMMButton()
	if CookyChat.db.profile.icon.hide then
		icon:Hide("CookyChat")
	else
		icon:Show("CookyChat")
	end
	
	if CookyChat.db.profile.semiOnOff and not CookyChat.db.profile.globalActive then
		CookyChatLDB.icon = "Interface\\AddOns\\CookyChat\\Media\\icon_r"
	elseif CookyChat.db.profile.globalActive then
		CookyChatLDB.icon = "Interface\\AddOns\\CookyChat\\Media\\icon"
	else
		CookyChatLDB.icon = "Interface\\AddOns\\CookyChat\\Media\\icon_bw"
	end
end

-- All different events to trigger off of and reply
function CookyChat:RegisterEvents()
	-- Chat messages
	CookyChatHandler:RegisterEvent("CHAT_MSG_BN_WHISPER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_SAY")
	CookyChatHandler:RegisterEvent("CHAT_MSG_GUILD")
	CookyChatHandler:RegisterEvent("CHAT_MSG_OFFICER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_PARTY")
	CookyChatHandler:RegisterEvent("CHAT_MSG_PARTY_LEADER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_RAID")
	CookyChatHandler:RegisterEvent("CHAT_MSG_RAID_LEADER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_RAID_WARNING")
	CookyChatHandler:RegisterEvent("CHAT_MSG_WHISPER")
	CookyChatHandler:RegisterEvent("CHAT_MSG_YELL")
	
	-- Achievements
	CookyChatHandler:RegisterEvent("CHAT_MSG_GUILD_ACHIEVEMENT")
	CookyChatHandler:RegisterEvent("CHAT_MSG_ACHIEVEMENT")
	
	-- Used for Guild leaving/joining
	CookyChatHandler:RegisterEvent("CHAT_MSG_SYSTEM")
end

-- Default values
local defaults = {
	global = {
		version = 0,
	},
	profile = {
		trigger = {
		},
		reply = {
		},
		groupActive = {
		},
		groupCaseSensitive = {
		},
		perChannelActive = {
		},
		groupChannelShow = {
		},
		groupChannelActive = {
		},
		groupTriggerType = {
		},
		otherChannelActive = {
		},
		otherChannelReplyShow = {
		},
		otherChannelReplyActiveGroup = {
		},
		whitelist = {
		},
		exactMatch = {
		},
		groupDelay = {
		},
		groupPercentage = {
		},
		
		suppressChat = false,
		testing = false,
		semiOnOff = false,
		globalWhitelist = "",
		blacklist = "",
		globalActive = true,
		numFilled = 0,
		spamAmount = 3,
		icon = {hide = false, radius = 80, minimapPos = 45},
		menuTree = {
		{
			value = "mainSettings",
			text = "Addon Settings",
		},
		{
			value = "groupSettings",
			text = "Groups",
			children =
			{
			}
		}
	}
	},
}

-- Defaults for various Channel values
local channelDefaults = {
	perChannelDefaultNames = {
		"Say",
		"Party",
		"Raid",
		"Guild",
		"Yell",
		"Whisper",
		"Battle.Net Whispers",
		"Officer",
	},
	
	otherChannelDefaultNames = {
		"Say",
		"Party",
		"Raid",
		"Guild",
		"Yell",
		"Officer",
		"Triggered Channel",
	},
	-- Defaults for Per Channel values
	perChannelDefaultValues = 
	{
		true, -- Say 1
		true, -- Party 2
		true, -- Raid 3
		true, -- Guild 4
		true, -- Yell 5
		true, -- Whisper 6
		true, -- Battle.Net Whispers 7
		true, -- Officer 8
	},
	
	-- Defaults for the Reply to Other Channels values
	otherChannelDefaultValues = 
	{
		false, -- Say 1
		false, -- Party 2
		false, -- Raid 3
		false, -- Guild 4
		false, -- Yell 5
		false, -- Officer 6
		false, -- Triggered Channel 7
	},
}

-- Tree for dropdown Trigger Type
local eventSelectTree = {
	["chatMessageSelect"] = "Chat Messages",
	["achievementSelect"] = "Achievements",
	["guildJoiningSelect"] = "Guild Joining",
	["guildLogInSelect"] = "Guild Logging In"
}

-- Alts for testing purpose
local alts = {
	"Icooky-Zul'jin",
	"Santhice-Zul'jin",
	"Feliaena-Zul'jin",
	"Zarina-Zul'jin",
	"Caimana-Aegwynn"
}

local unitName = UnitName("player")
local unitRealm = GetRealmName("player")
local unitFullName = unitName.."-"..unitRealm

function CookyChat:ReloadSettings()
	-- Minimap button.
	if icon and not icon:IsRegistered("CookyChat") then
		icon:Register("CookyChat", CookyChatLDB, CookyChat.db.profile.icon)
	end

	CookyChat:RefreshMMButton()
end

function CookyChat:OnEnable()
	CookyChat:ReloadSettings()
end

-- Initialize events
function CookyChat:OnInitialize()
	CookyChat.db = LibStub("AceDB-3.0"):New("CookyChatDB", defaults, true)
	SLASH_COOKY1 = "/cc"
	SLASH_COOKY2 = "/cookychat"
	
	function SlashCmdList.COOKY(msg, frame)
		local command, rest = msg:match("^(%S*)%s*(.-)$");
		if command == "minimap" then
			if rest == "on" then
				CookyChat.db.profile.icon.hide = false
				CookyChat:MinimapOnOrOff()
			elseif rest == "off" then
				CookyChat.db.profile.icon.hide = true
				CookyChat:MinimapOnOrOff()
			else
				CookyChat:MinimapOnOrOff(true)
			end
			CookyChat:RefreshMMButton()
		elseif command == "semi" then
			if rest == "on" then
				CookyChat.db.profile.semiOnOff = false
				CookyChat:SemiOnOrOff()
			elseif rest == "off" then
				CookyChat.db.profile.semiOnOff = true
				CookyChat:SemiOnOrOff()
			else
				CookyChat:SemiOnOrOff(true)
			end
			CookyChat:RefreshMMButton()
		elseif command == "open" then
			CookyChat:OpenGUI()
		elseif command == "on" then
			CookyChat.db.profile.globalActive = true
			CookyChat:OnOrOff()
			CookyChat:RefreshMMButton()
		elseif command == "off" then
			CookyChat.db.profile.globalActive = false
			CookyChat:OnOrOff()
			CookyChat:RefreshMMButton()
		elseif command == "help" then
			CookyChat:Print("Command Usage:")
			CookyChat:Print("|cffFF69B4/cc|cookychat|r |cffBF5FFFOn/Off|r to turn addon On/Off, or leave blank to toggle.")
			CookyChat:Print("|cffFF69B4/cc|cookychat|r |cffBF5FFFMiniMap: On/Off|r to turn minimap On/Off, or leave blank to toggle.")
			CookyChat:Print("|cffFF69B4/cc|cookychat|r |cffBF5FFFSemi: On/Off|r to turn semi-off mode On/Off, or leave blank to toggle.")
			CookyChat:Print("|cffFF69B4/cc|cookychat|r |cffBF5FFFOpen|r to open the gui.")
		else
			CookyChat:OnOrOff(true)
			CookyChat:RefreshMMButton()
		end
	end
	
	
	-- Handle version setting for new installers
	-- Transfer over old system variables to new system
	local updatedSaved = false
	if (CookyChat.db.global.version < 10) then
		for i = 1, CookyChat.db.profile.numFilled do
			CookyChat.db.profile.groupDelay[i] = 0
			CookyChat.db.profile.groupPercentage[i] = 100
		end
		updatedSaved = true
	end
	
	if (CookyChat.db.global.version < 9) then
		for i = 1, CookyChat.db.profile.numFilled do
			CookyChat.db.profile.otherChannelActive[i] = false
			CookyChat.db.profile.otherChannelReplyShow[i] = false
			CookyChat.db.profile.otherChannelReplyActiveGroup[i] = channelDefaults.otherChannelDefaultValues
		end
		updatedSaved = true
	end
	
	if (CookyChat.db.global.version < 7) then
		for i = 1, CookyChat.db.profile.numFilled do
			CookyChat.db.profile.groupChannelShow[i] = CookyChat.db.profile.hideChannels
			CookyChat.db.profile.perChannelActive[i] = CookyChat.db.profile.seperateChannelActivityOption
			CookyChat.db.profile.groupTriggerType[i] = "chatMessageSelect"
			CookyChat.db.profile.whitelist[i] = ""
			CookyChat.db.profile.exactMatch[i] = false
			table.insert(CookyChat.db.profile.menuTree[2].children, {value = "group"..i, text = "Group "..i})
		end
		CookyChat.db.profile.hideChannels = nil
		CookyChat.db.profile.seperateChannelActivityOption = nil
		CookyChat.db.profile.exactMode = nil
		updatedSaved = true
	end
	
	if (CookyChat.db.global.version ~= CURRENT_VERSION) then
		CookyChat.db.global.version = CURRENT_VERSION
	end
	
	if updatedSaved then
		C_Timer.After(2, function() 		
			CookyChat:Print('Saved Settings changed from old version to newer version, a reload using "/reload" is suggested to make sure everything is saved and works as intended.')
		end)
	end
	
	CookyChat:RegisterEvents()
end

local DisabledPerChannelActiveAchivements = {
	false, -- Say 1
	true, -- Party 2
	true, -- Raid 3
	false, -- Guild 4
	true, -- Yell 5
	true, -- Whisper 6
	true, -- Battle.Net Whispers 7
	true, -- Officer 8
}
local DisabledPerChannelActiveGuild = {
	true, -- Say 1
	true, -- Party 2
	true, -- Raid 3
	false, -- Guild 4
	true, -- Yell 5
	true, -- Whisper 6
	true, -- Battle.Net Whispers 7
	true, -- Officer 8
	}

-- Function to draw groups ui
local function DrawGroups(container, i)
	local function showChatWidgets()
		if CookyChat.db.profile.groupTriggerType[i] == "chatMessageSelect" then 
			return false 
		else 
			return true 
		end
	end
	
	local function showGuildWidgets()
		if CookyChat.db.profile.groupTriggerType[i] == "achievementSelect" or CookyChat.db.profile.groupTriggerType[i] == "chatMessageSelect" then 
			return false 
		else 
			return true 
		end
	end
	
	local scroll = AceGUI:Create("ScrollFrame")
	scroll:SetLayout("Flow")
	scroll:SetFullHeight(true)
	scroll:SetFullWidth(true)
	container:AddChild(scroll)
	
	-- Header
	local header = AceGUI:Create("Heading")
	header:SetText("Group: " ..CookyChat.db.profile.menuTree[2].children[i].text)
	header:SetFullWidth(true)
	scroll:AddChild(header)
	
	local inlineGroup1 = AceGUI:Create("InlineGroup")
	inlineGroup1:SetLayout("Flow")
	inlineGroup1:SetFullWidth(true)
	scroll:AddChild(inlineGroup1)
	
	-- Box for inputting group name
	local groupNameBox = AceGUI:Create("EditBox")
	groupNameBox:SetText(CookyChat.db.profile.menuTree[2].children[i].text)
	groupNameBox:SetRelativeWidth(0.66)
	groupNameBox:DisableButton(false)
	groupNameBox:SetLabel("Group Name")
	groupNameBox:SetCallback("OnEnterPressed",
		function(widget, event, text)
			CookyChat.db.profile.menuTree[2].children[i].text = text
			treewidget:SelectByPath("groupSettings?group"..i)
		end)
	groupNameBox:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Group Name", 1, .82, 0, true)
			GameTooltip:AddLine('Edit the name of the group to be shown in the tree to the left and header above.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	groupNameBox:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(groupNameBox)
	
	-- Dropdown Trigger Type
	local eventSelectDropdown = AceGUI:Create("Dropdown")
	eventSelectDropdown:SetList(eventSelectTree)
	eventSelectDropdown:SetRelativeWidth(0.33)
	eventSelectDropdown:SetLabel("Trigger Type Selector")
	eventSelectDropdown:SetValue(CookyChat.db.profile.groupTriggerType[i])
	eventSelectDropdown:SetMultiselect(false)
	eventSelectDropdown:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Trigger Type Selector", 1, .82, 0, true)
			GameTooltip:AddLine('Select which type of messages you want to reply to.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	eventSelectDropdown:SetCallback("OnValueChanged",
		function(widget, event, key)
			CookyChat.db.profile.groupTriggerType[i] = key
			treewidget:SelectByPath("groupSettings?group"..i)
		end)
	eventSelectDropdown:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(eventSelectDropdown)
	
	-- Group active toggle
	local groupActiveCheck = AceGUI:Create("CheckBox")
	groupActiveCheck:SetValue(CookyChat.db.profile.groupActive[i])
	groupActiveCheck:SetRelativeWidth(0.33)
	groupActiveCheck:SetLabel("Group Active")
	groupActiveCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.groupActive[i] = value or false
		end)
	groupActiveCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Group Toggle On/Off", 1, .82, 0, true)
			GameTooltip:AddLine('Toggles this group On or Off.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	groupActiveCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(groupActiveCheck)
	
	-- Case sensitive toggle
	local caseSensitiveCheck = AceGUI:Create("CheckBox")
	caseSensitiveCheck:SetRelativeWidth(0.33)
	caseSensitiveCheck:SetValue(CookyChat.db.profile.groupCaseSensitive[i])
	caseSensitiveCheck:SetLabel("Case Sensitive")
	caseSensitiveCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.groupCaseSensitive[i] = value or false
		end)
	caseSensitiveCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Case Sensitive", 1, .82, 0, true)
			GameTooltip:AddLine("Turns case sensitivity on or off for this group", 1, 1, 1, true)
			GameTooltip:Show()
		end)
	caseSensitiveCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(caseSensitiveCheck)
	
	-- Exact Match checkbox
	local exactMatchCheck = AceGUI:Create("CheckBox")
	exactMatchCheck:SetRelativeWidth(0.33)
	exactMatchCheck:SetValue(CookyChat.db.profile.exactMatch[i] or false)
	exactMatchCheck:SetLabel("Exact match")
	exactMatchCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.exactMatch[i] = value or false
		end)
	exactMatchCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Case Sensitive", 1, .82, 0, true)
			GameTooltip:AddLine('The trigger must be matched exactly, no additional letters or symbols around the word. I.E. if "test" was the trigger, "1test" would not cause the reply.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	exactMatchCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(exactMatchCheck)
	
	-- Group Delay
	local delaySlider = AceGUI:Create("Slider")
	delaySlider:SetValue(CookyChat.db.profile.groupDelay[i])
	delaySlider:SetSliderValues(0, 60, 1)
	delaySlider:SetLabel("Reply Delay")
	delaySlider:SetRelativeWidth(0.5)
	delaySlider:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Reply Delay", 1, .82, 0, true)
			GameTooltip:AddLine('Delay (in seconds) before the reply is sent, set to 0 for instant reply.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	delaySlider:SetCallback("OnMouseUp",
		function(widget, event, value)
			CookyChat.db.profile.groupDelay[i] = value
		end)
	delaySlider:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(delaySlider)
	
	-- Group Reply Percentage
	local percentageSlider = AceGUI:Create("Slider")
	percentageSlider:SetValue(CookyChat.db.profile.groupPercentage[i])
	percentageSlider:SetSliderValues(1, 100, 1)
	percentageSlider:SetLabel("Reply Percentage")
	percentageSlider:SetRelativeWidth(0.5)
	percentageSlider:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Reply Percentage", 1, .82, 0, true)
			GameTooltip:AddLine('Percentage chance a reply has of being sent, set to 100 to reply every time.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	percentageSlider:SetCallback("OnMouseUp",
		function(widget, event, value)
			CookyChat.db.profile.groupPercentage[i] = value
		end)
	percentageSlider:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(percentageSlider)
	
	-- Per Channel Activity check
	local perChannelCheck = AceGUI:Create("CheckBox")
	perChannelCheck:SetValue(CookyChat.db.profile.perChannelActive[i])
	perChannelCheck:SetLabel("Per Channel Activity")
	perChannelCheck:SetDisabled(showGuildWidgets())
	perChannelCheck:SetRelativeWidth(0.5)
	perChannelCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.perChannelActive[i] = value or false
			if value then
				-- Show the "show active" checkbox and reselect the current group
				inlineGroup1:AddChild(showActiveCheck)
				treewidget:SelectByPath("groupSettings?group"..i)
			else
				-- Remove the "show active" checkbox and reselect the current group
				showActiveCheck:ClearAllPoints()
				treewidget:SelectByPath("groupSettings?group"..i)
			end
		end)
	perChannelCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Per Channel Activity", 1, .82, 0, true)
			GameTooltip:AddLine('Allows trigger groups to work on a per channel basis.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	perChannelCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(perChannelCheck)
	
	-- "Show Active" checkbox
	showActiveCheck = AceGUI:Create("CheckBox")
	showActiveCheck:SetRelativeWidth(0.5)
	showActiveCheck:SetValue(CookyChat.db.profile.groupChannelShow[i])
	showActiveCheck:SetLabel("Show the Active Channels")
	showActiveCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.groupChannelShow[i] = value or false
			if value then
				-- Show the channel checkboxes and reselect current group
				scroll:AddChild(perChannelInlineGroup)
				treewidget:SelectByPath("groupSettings?group"..i)
			else
				-- Hide the channel checkboxes and reselect current group
				perChannelInlineGroup:ClearAllPoints()
				treewidget:SelectByPath("groupSettings?group"..i)
			end
		end)
	showActiveCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Show the Active Channels", 1, .82, 0, true)
			GameTooltip:AddLine('Hide/Show the "Active Channels" selection. The trigger will still reply on only the selected channels, this merely hides the selection of checkboxes', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	showActiveCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	if CookyChat.db.profile.perChannelActive[i] then
		inlineGroup1:AddChild(showActiveCheck)
	end
	
	-- Other Channel Reply Activity check
	local otherReplyChannelCheck = AceGUI:Create("CheckBox")
	otherReplyChannelCheck:SetValue(CookyChat.db.profile.otherChannelActive[i])
	otherReplyChannelCheck:SetLabel("Other Channel Reply Activity")
	otherReplyChannelCheck:SetRelativeWidth(0.5)
	otherReplyChannelCheck:SetDisabled(showChatWidgets())
	otherReplyChannelCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.otherChannelActive[i] = value or false
			if value then
				-- Show the "show active" checkbox and reselect the current group
				inlineGroup1:AddChild(showOtherReplyCheck)
				treewidget:SelectByPath("groupSettings?group"..i)
			else
				-- Remove the "show active" checkbox and reselect the current group
				showOtherReplyCheck:ClearAllPoints()
				treewidget:SelectByPath("groupSettings?group"..i)
			end
		end)
	otherReplyChannelCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Other Channel Reply Activity", 1, .82, 0, true)
			GameTooltip:AddLine('Allows replies to be sent into other channels, not just where the trigger was seen.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	otherReplyChannelCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(otherReplyChannelCheck)
	
	-- "Show Other Group Reply Active" checkbox
	showOtherReplyCheck = AceGUI:Create("CheckBox")
	showOtherReplyCheck:SetRelativeWidth(0.5)
	showOtherReplyCheck:SetValue(CookyChat.db.profile.otherChannelReplyShow[i])
	showOtherReplyCheck:SetLabel("Show the Other Reply Channels")
	showOtherReplyCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.otherChannelReplyShow[i] = value or false
			if value then
				-- Show the channel checkboxes and reselect current group
				scroll:AddChild(otherReplyInlineGroup)
				treewidget:SelectByPath("groupSettings?group"..i)
			else
				-- Hide the channel checkboxes and reselect current group
				otherReplyInlineGroup:ClearAllPoints()
				treewidget:SelectByPath("groupSettings?group"..i)
			end
		end)
	showOtherReplyCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Show the Other Reply Channels", 1, .82, 0, true)
			GameTooltip:AddLine('Hide/Show the "Other Reply Channels" selection. The trigger will still reply on only the selected channels, this merely hides the selection of checkboxes', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	showOtherReplyCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	if CookyChat.db.profile.otherChannelActive[i] then
		inlineGroup1:AddChild(showOtherReplyCheck)
	end 
	
	-- Box for inputting trigger
	local triggerBox = AceGUI:Create("EditBox")
	triggerBox:SetLabel("Trigger")
	triggerBox:SetText(CookyChat.db.profile.trigger[i])
	triggerBox:SetDisabled(showChatWidgets())
	triggerBox:SetFullWidth(true)
	triggerBox:DisableButton(true)
	triggerBox:SetCallback("OnTextChanged", 
		function(widget, event, text) 
			CookyChat.db.profile.trigger[i] = text 
		end)
	triggerBox:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Trigger", 1, .82, 0, true)
			GameTooltip:AddLine("The message that will trigger the autoreply.", 1, 1, 1, true)
			GameTooltip:Show()
		end)
	triggerBox:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	scroll:AddChild(triggerBox)
	
	-- Deciding which tooltip to display for the reply box based off trigger type
	local tooltip
	
	if CookyChat.db.profile.groupTriggerType[i] == "achievementSelect" then
		tooltip = achivementReplyTooltip
	elseif CookyChat.db.profile.groupTriggerType[i] == "guildJoiningSelect" then
		tooltip = guildReplyTooltip
	else
		tooltip = chatReplyTooltip
	end
	
	-- Box for inputting reply
	local replyBox = AceGUI:Create("EditBox")
	replyBox:SetLabel("Reply")
	replyBox:SetText(CookyChat.db.profile.reply[i])
	replyBox:SetFullWidth(true)
	replyBox:DisableButton(true)
	replyBox:SetCallback("OnTextChanged", function(widget, event, text) CookyChat.db.profile.reply[i] = text end)
	replyBox:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Reply", 1, .82, 0, true)
			GameTooltip:AddLine(tooltip, 1, 1, 1, true)
			GameTooltip:Show()
		end)
	replyBox:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	scroll:AddChild(replyBox)
	
	-- Box for inputting whitelist
	local whitelistBox = AceGUI:Create("EditBox")
	whitelistBox:SetLabel("Group Whitelist")
	whitelistBox:SetText(CookyChat.db.profile.whitelist[i])
	whitelistBox:SetFullWidth(true)
	whitelistBox:DisableButton(true)
	whitelistBox:SetCallback("OnTextChanged", function(widget, event, text) CookyChat.db.profile.whitelist[i] = text end)
	whitelistBox:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Group Whitelist", 1, .82, 0, true)
			GameTooltip:AddLine("Insert character names of specific people you want this group to reply to, sperated by spaces. If left empty, this group will reply to anyone who triggers it. Will always reply to BNet whispers.", 1, 1, 1, true)
			GameTooltip:Show()
		end)
	whitelistBox:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	scroll:AddChild(whitelistBox)
	
	-- Group box for per channel checkboxes
	perChannelInlineGroup = AceGUI:Create("InlineGroup")
	perChannelInlineGroup:SetTitle("Active Channels")
	perChannelInlineGroup:SetLayout("Flow")
	perChannelInlineGroup:SetFullWidth(true)
	if CookyChat.db.profile.groupChannelShow[i] and CookyChat.db.profile.perChannelActive[i] then
		scroll:AddChild(perChannelInlineGroup)
	end
	
	-- The 8 checkboxes for individual channels selections
	for j = 1, 8 do
		local function showWidget()
			if CookyChat.db.profile.groupTriggerType[i] == "achievementSelect" then 
				return DisabledPerChannelActiveAchivements[j]
			elseif (CookyChat.db.profile.groupTriggerType[i] == "guildJoiningSelect" or CookyChat.db.profile.groupTriggerType[i] == "guildLogInSelect") then
				return DisabledPerChannelActiveGuild[j]
			end
		end
		
		local check = AceGUI:Create("CheckBox")
		check:SetRelativeWidth(0.5)
		check:SetLabel(channelDefaults.perChannelDefaultNames[j])
		check:SetValue(CookyChat.db.profile.groupChannelActive[i][j])
		check:SetDisabled(showWidget())
		check:SetCallback("OnValueChanged",
			function(widget, event, value)
				CookyChat.db.profile.groupChannelActive[i][j] = value or false
			end)
		check:SetCallback("OnEnter",
			function(widget)
				GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
				GameTooltip:SetText(channelDefaults.perChannelDefaultNames[j], 1, .82, 0, true)
				GameTooltip:Show()
			end)
		check:SetCallback("OnLeave",
			function()
				GameTooltip:Hide()
			end)
		perChannelInlineGroup:AddChild(check)
	end
	
	-- Group box for other channel reply checkboxes
	otherReplyInlineGroup = AceGUI:Create("InlineGroup")
	otherReplyInlineGroup:SetTitle("Other Reply Channels")
	otherReplyInlineGroup:SetLayout("Flow")
	otherReplyInlineGroup:SetFullWidth(true)
	if CookyChat.db.profile.otherChannelActive[i] and CookyChat.db.profile.otherChannelReplyShow[i] then
		scroll:AddChild(otherReplyInlineGroup)
	end
	
	-- The 7 checkboxes for individual channels selections
	for j = 1, 7 do
		local check = AceGUI:Create("CheckBox")
		check:SetRelativeWidth(0.5)
		check:SetLabel(channelDefaults.otherChannelDefaultNames[j])
		check:SetValue(CookyChat.db.profile.otherChannelReplyActiveGroup[i][j])
		check:SetCallback("OnValueChanged",
			function(widget, event, value)
				CookyChat.db.profile.otherChannelReplyActiveGroup[i][j] = value or false
			end)
		check:SetCallback("OnEnter",
			function(widget)
				GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
				GameTooltip:SetText(channelDefaults.otherChannelDefaultNames[j], 1, .82, 0, true)
				GameTooltip:Show()
			end)
		check:SetCallback("OnLeave",
			function()
				GameTooltip:Hide()
			end)
		otherReplyInlineGroup:AddChild(check)
	end
	
	-- Button for deleting the group
	local deleteButton = AceGUI:Create("Button")
	deleteButton:SetText("Delete Group")
	deleteButton:SetRelativeWidth(1)
	deleteButton:SetCallback("OnClick",
		function()
			-- Selection = what group to select next after deleting current
			local selection
			if i == CookyChat.db.profile.numFilled and i == 1 then
				selection = "mainSettings"
			elseif i == CookyChat.db.profile.numFilled then
				selection = "groupSettings?group"..(i-1)
			else 
				selection = "groupSettings?group"..i
			end
		
			-- Edit all group values above one being deleted to keep in numerical order
			-- Or else it breaks the selection based off value name (it goes off the number
			-- at the end: group1, group2, group3)
			for j = i, CookyChat.db.profile.numFilled do
				local num = tonumber(strsub(CookyChat.db.profile.menuTree[2].children[j].value, 6)) - 1
				CookyChat.db.profile.menuTree[2].children[j].value = "group" ..num
			end
			
			CookyChat.db.profile.numFilled = CookyChat.db.profile.numFilled - 1
			
			-- Remove values from saved variables
			table.remove(CookyChat.db.profile.menuTree[2].children, i)
			table.remove(CookyChat.db.profile.trigger, i)
			table.remove(CookyChat.db.profile.reply, i)
			table.remove(CookyChat.db.profile.groupActive, i)
			table.remove(CookyChat.db.profile.groupCaseSensitive, i)
			table.remove(CookyChat.db.profile.perChannelActive, i)
			table.remove(CookyChat.db.profile.groupChannelShow, i)
			table.remove(CookyChat.db.profile.groupChannelActive, i)
			table.remove(CookyChat.db.profile.otherChannelReplyShow, i)
			table.remove(CookyChat.db.profile.otherChannelReplyActiveGroup, i)
			table.remove(CookyChat.db.profile.groupTriggerType, i)
			table.remove(CookyChat.db.profile.whitelist, i)
			table.remove(CookyChat.db.profile.exactMatch, i)
			table.remove(CookyChat.db.profile.groupDelay, i)
			table.remove(CookyChat.db.profile.groupPercentage, i)
			
			treewidget:SelectByPath(selection)
		end)
	deleteButton:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Delete Group", 1, .82, 0, true)
			GameTooltip:AddLine("Delete this group.", 1, 1, 1, true)
			GameTooltip:Show()
		end)
	deleteButton:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	scroll:AddChild(deleteButton)
end

-- Function to draw main settings ui
local function DrawMainSettings(container)
	-- Scroll Frame
	local scroll = AceGUI:Create("ScrollFrame")
	scroll:SetLayout("Flow")
	scroll:SetFullHeight(true)
	scroll:SetFullWidth(true)
	container:AddChild(scroll)
	
	-- "Addon Settings" Header
	local header = AceGUI:Create("Heading")
	header:SetText("Addon Settings")
	header:SetFullWidth(true)
	scroll:AddChild(header)
	
	-- Group for addon/tooltip/ settings and modes
	local inlineGroup1 = AceGUI:Create("InlineGroup")
	inlineGroup1:SetLayout("Flow")
	inlineGroup1:SetFullWidth(true)
	scroll:AddChild(inlineGroup1)
	
	-- Toggle for addon on or off
	local addonActiveCheck = AceGUI:Create("CheckBox")
	addonActiveCheck:SetValue(CookyChat.db.profile.globalActive)
	addonActiveCheck:SetLabel("Addon Toggle On/Off")
	addonActiveCheck:SetRelativeWidth(0.5)
	addonActiveCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.globalActive = value or false
			CookyChat:OnOrOff()
			CookyChat:RefreshMMButton()
		end)
	addonActiveCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Addon Toggle On/Off", 1, .82, 0, true)
			GameTooltip:AddLine('Toggles the addon On or Off.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	addonActiveCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(addonActiveCheck)
	
	-- Toggle for minimap icon on or off
	local minimapIconActiveCheck = AceGUI:Create("CheckBox")
	minimapIconActiveCheck:SetValue(not CookyChat.db.profile.icon.hide)
	minimapIconActiveCheck:SetLabel("Minimap Icon Toggle On/Off")
	minimapIconActiveCheck:SetRelativeWidth(0.5)
	minimapIconActiveCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			if value then 
				CookyChat.db.profile.icon.hide = false
			else
				CookyChat.db.profile.icon.hide = true
			end
			CookyChat:MinimapOnOrOff()
			CookyChat:RefreshMMButton()
		end)
	minimapIconActiveCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Minimap Icon Toggle On/Off", 1, .82, 0, true)
			GameTooltip:AddLine('Toggles the minimap icon On or Off.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	minimapIconActiveCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(minimapIconActiveCheck)
	
	-- Toggle for semi-off mode
	local semiCheck = AceGUI:Create("CheckBox")
	semiCheck:SetValue(CookyChat.db.profile.semiOnOff)
	semiCheck:SetLabel("Semi-Off mode")
	semiCheck:SetRelativeWidth(0.5)
	semiCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.semiOnOff = value or false
			CookyChat:SemiOnOrOff()
			CookyChat:RefreshMMButton()
		end)
	semiCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Semi-Off mode", 1, .82, 0, true)
			GameTooltip:AddLine('People in the "Global Whitelist" will still be replied to even if the addon is turned off.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	semiCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(semiCheck)
	
	-- Toggle for suppressing chat notifications for setting changes
	local suppressCheck = AceGUI:Create("CheckBox")
	suppressCheck:SetValue(CookyChat.db.profile.suppressChat)
	suppressCheck:SetLabel("Suppress setting messages")
	suppressCheck:SetRelativeWidth(0.5)
	suppressCheck:SetCallback("OnValueChanged",
		function(widget, event, value)
			CookyChat.db.profile.suppressChat = value or false
		end)
	suppressCheck:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Suppress setting messages", 1, .82, 0, true)
			GameTooltip:AddLine('Suppresses messages in chat about settings turning On or Off.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	suppressCheck:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(suppressCheck)
	
	-- Slider for spam value
	local spamSlider = AceGUI:Create("Slider")
	spamSlider:SetValue(CookyChat.db.profile.spamAmount)
	spamSlider:SetSliderValues(1, 10, 1)
	spamSlider:SetLabel("Maximum spam amount")
	spamSlider:SetRelativeWidth(0.5)
	spamSlider:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Maximum spam amount", 1, .82, 0, true)
			GameTooltip:AddLine('Maximum amount of messages to send in a 10 second time frame.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	spamSlider:SetCallback("OnMouseUp",
		function(widget, event, value)
			CookyChat.db.profile.spamAmount = value
		end)
	spamSlider:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(spamSlider)
	
	-- Button to create a new group
	local createGroupButton = AceGUI:Create("Button")
	createGroupButton:SetText("Create New Group")
	createGroupButton:SetRelativeWidth(1)
	createGroupButton:SetCallback("OnClick",
		function()
			CookyChat.db.profile.numFilled = CookyChat.db.profile.numFilled + 1
			
			-- Insert settings for new group at the end of current chain
			table.insert(CookyChat.db.profile.menuTree[2].children,{value = "group"..CookyChat.db.profile.numFilled, text = "Group "..CookyChat.db.profile.numFilled})
			table.insert(CookyChat.db.profile.trigger, "")
			table.insert(CookyChat.db.profile.reply, "")
			table.insert(CookyChat.db.profile.groupActive, true)
			table.insert(CookyChat.db.profile.groupCaseSensitive, false)
			table.insert(CookyChat.db.profile.perChannelActive, false)
			table.insert(CookyChat.db.profile.groupChannelShow, false)
			table.insert(CookyChat.db.profile.groupChannelActive, channelDefaults.perChannelDefaultValues)
			table.insert(CookyChat.db.profile.otherChannelReplyShow, false)
			table.insert(CookyChat.db.profile.otherChannelReplyActiveGroup, channelDefaults.otherChannelDefaultValues)
			table.insert(CookyChat.db.profile.groupTriggerType,"chatMessageSelect")
			table.insert(CookyChat.db.profile.whitelist, "")
			table.insert(CookyChat.db.profile.exactMatch, false)
			table.insert(CookyChat.db.profile.groupDelay, 0)
			table.insert(CookyChat.db.profile.groupPercentage, 100)
			
			treewidget:SelectByPath("groupSettings?group"..CookyChat.db.profile.numFilled)
			CookyChat:Print('New group created, a reload using "/reload" is suggested to make sure the "Per Channel" and "Other Channel Reply" activities are saved properly.')
		end)
	createGroupButton:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Create New Group", 1, .82, 0, true)
			GameTooltip:AddLine("Create a new group.", 1, 1, 1, true)
			GameTooltip:Show()
		end)
	createGroupButton:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	inlineGroup1:AddChild(createGroupButton)
	
	-- Whitelist input box
	local globalWhitelistBox = AceGUI:Create("EditBox")
	globalWhitelistBox:SetLabel("Global Whitelist")
	globalWhitelistBox:SetText(CookyChat.db.profile.globalWhitelist)
	globalWhitelistBox:SetFullWidth(true)
	globalWhitelistBox:DisableButton(true)
	globalWhitelistBox:SetCallback("OnTextChanged", 
		function(widget, event, text) 
			CookyChat.db.profile.globalWhitelist = text 
		end)
	globalWhitelistBox:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Global Whitelist", 1, .82, 0, true)
			GameTooltip:AddLine('Insert character names that you want to whitelist, seperated by spaces. Character names in this box will still be replied to in "Semi-Off" mode.', 1, 1, 1, true)
			GameTooltip:Show()
		end)
	globalWhitelistBox:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	scroll:AddChild(globalWhitelistBox)
	
	-- Blacklist input box
	local blacklistBox = AceGUI:Create("EditBox")
	blacklistBox:SetLabel("Blacklist")
	blacklistBox:SetText(CookyChat.db.profile.blacklist)
	blacklistBox:SetFullWidth(true)
	blacklistBox:DisableButton(true)
	blacklistBox:SetCallback("OnTextChanged", 
		function(widget, event, text) 
			CookyChat.db.profile.blacklist = text 
		end)
	blacklistBox:SetCallback("OnEnter",
		function(widget)
			GameTooltip:SetOwner(widget.frame, "ANCHOR_TOPLEFT")
			GameTooltip:SetText("Blacklist", 1, .82, 0, true)
			GameTooltip:AddLine("Insert character names that you want to never reply to, seperated by spaces. Can't blacklist BNet names.", 1, 1, 1, true)
			GameTooltip:Show()
		end)
	blacklistBox:SetCallback("OnLeave",
		function()
			GameTooltip:Hide()
		end)
	scroll:AddChild(blacklistBox)
	
	for i = 1, #alts do
		if unitFullName == alts[i] then
			--Toggle for testing
			local testingCheck = AceGUI:Create("CheckBox")
			testingCheck:SetValue(CookyChat.db.profile.testing)
			testingCheck:SetLabel("Testing")
			testingCheck:SetRelativeWidth(0.5)
			testingCheck:SetCallback("OnValueChanged",
				function(widget, event, value)
					CookyChat.db.profile.testing = value or false
				end)
			inlineGroup1:AddChild(testingCheck)
		end
	end
end

-- Function to tell what group to decide on clicking the tree
local function SelectTree(container, event, group)
	-- If select "Group" and there's atleast 1 group, select group 10
	-- Else, select main addon settings frame
	local selection
	if CookyChat.db.profile.numFilled == 0 then
		selection = "mainSettings"	
	else
		selection = "groupSettings?group1"
	end
	
	container:ReleaseChildren()
	if group == "mainSettings" then
		DrawMainSettings(container)
	elseif group == "groupSettings" then
		treewidget:SelectByPath(selection)
	elseif strsub(group, 15, 19) == "group" then
		local num = tonumber(strsub(group, 20))
		DrawGroups(container, num)
	end
end

-- Function to tell the tooltip when you mouse over a group what to show
local function MouseOverTree(widget, event, path, frame)
	if strsub(path, 15, 19) == "group" then
		local num = tonumber(strsub(path, 20))
		local fillerAdded = false
		local active
		
		GameTooltip:SetOwner(frame, "ANCHOR_TOPLEFT")
		
		-- Group Active
		if CookyChat.db.profile.groupActive[num] then
			active = "On"
		else
			active = "Off"
		end
		
		GameTooltip:AddLine("|cffFFD100Group|cffFFFFFF: "..active.."|r", nil, nil, nil, true)
		
		-- Trigger Type
		GameTooltip:AddLine("|cffFFD100Trigger Type|cffFFFFFF: "..eventSelectTree[CookyChat.db.profile.groupTriggerType[num]].."|r", nil, nil, nil, true)
		
		-- Trigger
		if CookyChat.db.profile.groupTriggerType[num] == "chatMessageSelect" then
			GameTooltip:AddLine("|cffFFD100Trigger|cffFFFFFF: "..CookyChat.db.profile.trigger[num].."|r", nil, nil, nil, true)
		end
		
		-- Reply
		GameTooltip:AddLine("|cffFFD100Reply|cffFFFFFF: "..CookyChat.db.profile.reply[num].."|r", nil, nil, nil, true)
		
		-- Exact Match
		if CookyChat.db.profile.exactMatch[num] then
			active = "On"
		else
			active = "Off"
		end
		
		GameTooltip:AddLine("|cffFFD100Exact Match|cffFFFFFF: "..active.."|r", nil, nil, nil, true)
		
		-- Case Sensitive
		if CookyChat.db.profile.groupCaseSensitive[num] then
			active = "On"
		else
			active = "Off"
		end
		
		GameTooltip:AddLine("|cffFFD100Case Sensitive|cffFFFFFF: "..active.."|r", nil, nil, nil, true)
		
		-- Reply Delay
		GameTooltip:AddLine("|cffFFD100Reply Delay|cffFFFFFF: "..CookyChat.db.profile.groupDelay[num].."|r", nil, nil, nil, true)
		
		-- Reply Percentage
		GameTooltip:AddLine("|cffFFD100Reply Percentage|cffFFFFFF: "..CookyChat.db.profile.groupPercentage[num].."|r", nil, nil, nil, true)
		
		-- Per Channel Active
		local text = "|cffFFD100Per Channel Active|cffFFFFFF: "
		
		if CookyChat.db.profile.perChannelActive[num] then
			
			for i = 1, 8 do
				if CookyChat.db.profile.groupChannelActive[num][i] then
					text = text..channelDefaults.perChannelDefaultNames[i]..", "
					fillerAdded = true
				end
			end
			
			if fillerAdded then
				text = text:sub(1,-3)
			end
		else 
			text = text.."Off"
		end
		
		GameTooltip:AddLine(text.."|r", nil, nil, nil, true)
		
		-- Other Channel Reply Active
		text = "|cffFFD100Other Channel Reply Active|cffFFFFFF: "
			
		if CookyChat.db.profile.otherChannelActive[num] then
			local fillerAdded = false
			for i = 1, 7 do
				if CookyChat.db.profile.otherChannelReplyActiveGroup[num][i] then
					text = text..channelDefaults.otherChannelDefaultNames[i]..", "
					fillerAdded = true
				end
			end
			
			if fillerAdded then
				text = text:sub(1,-3)
			end
		else 
			text = text.."Off"
		end
		
		GameTooltip:AddLine(text.."|r", nil, nil, nil, true)
		
		-- Group Whitelist
		GameTooltip:AddLine("|cffFFD100Whitelist|cffFFFFFF: "..CookyChat.db.profile.whitelist[num].."|r", nil, nil, nil, true)
		
		GameTooltip:Show()
	end
end

-- Create the frame container
function CookyChat:DrawFrame()
	-- Main frame
	local frame = AceGUI:Create("Frame")
	frame:SetTitle("CookyChat v"..CURRENT_VERSION)
	frame:SetCallback("OnClose", function(widget) frameOpen = false AceGUI:Release(widget) end)
	frame:SetFullHeight(true)
	frame:SetLayout("Fill")
	
	-- Tree widget for categorizing
	treewidget = AceGUI:Create("TreeGroup")
	treewidget:SetTree(CookyChat.db.profile.menuTree)
	treewidget:SetLayout("Fill")
	treewidget:SetFullWidth(true)
	treewidget:SetFullHeight(true)
	treewidget:EnableButtonTooltips(false)
	treewidget:SetCallback("OnGroupSelected", SelectTree)
	treewidget:SetCallback("OnButtonEnter", MouseOverTree)
	treewidget:SetCallback("OnButtonLeave",
		function()
			GameTooltip:Hide()
		end)
	frame:AddChild(treewidget)
	
	-- This just expands the tree when you open the window
	treewidget:SelectByPath("groupSettings")
	-- Then goes back to main addon settings
	treewidget:SelectByPath("mainSettings")
end

-- Open the gui
function CookyChat:OpenGUI()
	if not frameOpen then 
			CookyChat:DrawFrame()
		frameOpen = true 
	end 
end

-- When mouseover the minimap icon
function CookyChatLDB:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_NONE")
	GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
	GameTooltip:ClearLines()
	GameTooltip:SetText("|cffBF5FFFCooky|rChat", 1, .82, 1, true)
	GameTooltip:AddLine("|cffFF69B4Left-Click|r: toggle On/Off.", 1, 1, 1, 1)
	GameTooltip:AddLine('|cffFF69B4Shift Left-Click|r: toggle "Semi-Off" mode On/Off.', 1, 1, 1, 1)
	GameTooltip:AddLine('|cffFF69B4Right-Click|r: open settings menu.', 1, 1, 1, 1)
	GameTooltip:Show()
end

-- On click functions for minimap button
function CookyChatLDB:OnClick(button)
	if button == "LeftButton" and IsShiftKeyDown() then
		CookyChat:SemiOnOrOff(true)
		CookyChat:RefreshMMButton()
	elseif button == "LeftButton" then
		CookyChat:OnOrOff(true)
		CookyChat:RefreshMMButton()
	elseif button == "RightButton" then
		CookyChat:OpenGUI()
	end
end

-- Semi-mode on or off toggling message
function CookyChat:SemiOnOrOff(toggle)
	local temp
	if toggle then
		self.db.profile.semiOnOff = not self.db.profile.semiOnOff
	end

	if not self.db.profile.suppressChat then
		if self.db.profile.semiOnOff then 
			temp = "On."
		else 
			temp = "Off." 
		end
		
		CookyChat:Print('Semi-Off mode |cffFF69B4'..temp)
	end
end

-- Addon on or off toggling message
function CookyChat:OnOrOff(toggle)
	local temp
	if toggle then
		self.db.profile.globalActive = not self.db.profile.globalActive
	end

	if not self.db.profile.suppressChat then
		if self.db.profile.globalActive then 
			temp = "On."
		else 
			temp = "Off." 
		end
		
		CookyChat:Print('|cffFF69B4'..temp)
	end
end

-- Minimap on or off toggling message
function CookyChat:MinimapOnOrOff(toggle)
	local temp
	if toggle then
		self.db.profile.icon.hide = not self.db.profile.icon.hide
	end
	
	if not self.db.profile.suppressChat then
		if not self.db.profile.icon.hide then 
			temp = "On."
		else 
			temp = "Off." 
		end
		
		CookyChat:Print('Minimap |cffFF69B4'..temp)
	end
end

function CookyChat:PercentageCheck(i)
	if math.random() <= (CookyChat.db.profile.groupPercentage[i] / 100) then
		return true
	else
		return false
	end
end

local spamBlock = false
local msgNumber = 0

-- Reply for chat messages
function CookyChat:ChatMsgReply(event, num, shortSource, fullSource, pID, otherChannelTable)
	-- Remove duplicate events with trigger, the source event, and a selected channel
	-- IE, if "say" and "trigger", and source is say, dont want to reply twice
	for i = 1, #otherChannelTable do
		if otherChannelTable[i] == "trigger" then
			for j = 1, #otherChannelTable do
				if otherChannelTable[j] == event then
					table.remove(otherChannelTable, i)
				end
			end
		end
	end
	
	local saveEvent = event
	
	for i = 1, #otherChannelTable do
		if otherChannelTable[i] == "trigger" then
			event = saveEvent
		else
			event = otherChannelTable[i]
		end
		
		C_Timer.After(10, function() 
			msgNumber = 0 
			spamBlock = false 
		end)
		
		msgNumber = msgNumber + 1
		
		if msgNumber == CookyChat.db.profile.spamAmount then
			spamBlock = true
		end
		
		reply = CookyChat.db.profile.reply[num]
		
		if string.find(reply, "%%S") then
			if event ~= "CHAT_MSG_WHISPER" and event ~= "CHAT_MSG_BN_WHISPER" then
				reply = string.gsub(reply, "%%S", shortSource)
			elseif event == "CHAT_MSG_WHISPER" then
				reply = string.gsub(reply, "%%S", fullSource)
			else
				reply = string.gsub(reply, "%%S", "")
			end
		end
		if string.find(reply, "%%F") then
			if event ~= "CHAT_MSG_BN_WHISPER" then
				reply = string.gsub(reply, "%%F", fullSource)
			else
				reply = string.gsub(reply, "%%F", "")
			end
		end
		
		if event == "CHAT_MSG_SAY" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then -- If message is from say and source isn't player
			if CookyChat.db.profile.perChannelActive[num] then -- If seperate channel per group and group is active
				if CookyChat.db.profile.groupChannelActive[num][1] then
					SendChatMessage(reply,"SAY")
				end
			else
				SendChatMessage(reply,"SAY")
			end
		elseif event == "CHAT_MSG_PARTY_LEADER" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then 
				if CookyChat.db.profile.groupChannelActive[num][2] then
					SendChatMessage(reply,"PARTY")
				end
			else
				SendChatMessage(reply,"PARTY")
			end	
		elseif event == "CHAT_MSG_PARTY" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then 
				if CookyChat.db.profile.groupChannelActive[num][2] then
					SendChatMessage(reply,"PARTY")
				end
			else
				SendChatMessage(reply,"PARTY")
			end	
		elseif event == "CHAT_MSG_RAID" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then 
				if CookyChat.db.profile.groupChannelActive[num][3] then
					SendChatMessage(reply,"RAID")
				end
			else
				SendChatMessage(reply,"RAID")
			end	
		elseif event == "CHAT_MSG_RAID_LEADER" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then 
				if CookyChat.db.profile.groupChannelActive[num][3] then
					SendChatMessage(reply,"RAID")
				end
			else
				SendChatMessage(reply,"RAID")
			end	
		elseif event == "CHAT_MSG_RAID_WARNING" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then 
				if CookyChat.db.profile.groupChannelActive[num][3] then
					SendChatMessage(reply,"RAID")
				end
			else
				SendChatMessage(reply,"RAID")
			end
		elseif event == "CHAT_MSG_GUILD" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then 
				if CookyChat.db.profile.groupChannelActive[num][4] then
					SendChatMessage(reply,"GUILD")
				end			
			else
				SendChatMessage(reply,"GUILD")
			end
		elseif event == "CHAT_MSG_YELL" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then
				if CookyChat.db.profile.groupChannelActive[num][5] then
					SendChatMessage(reply,"YELL")
				end
			else
				SendChatMessage(reply,"YELL")
			end
		elseif event == "CHAT_MSG_WHISPER" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then 
				if CookyChat.db.profile.groupChannelActive[num][6] then
					SendChatMessage(reply,"WHISPER", nil, fullSource)
				end
			else
				SendChatMessage(reply,"WHISPER", nil, fullSource)
			end
		elseif event == "CHAT_MSG_BN_WHISPER" then
			if not BNIsSelf(pID) then
				if CookyChat.db.profile.perChannelActive[num] then 
					if CookyChat.db.profile.groupChannelActive[num][7] then
						C_Timer.After(.15, function() BNSendWhisper(pID,reply) end)
					end
				else
					C_Timer.After(.15, function() BNSendWhisper(pID,reply) end)
				end
			end
		elseif event == "CHAT_MSG_OFFICER" and (CookyChat.db.profile.testing or shortSource ~= UnitName("player")) then
			if CookyChat.db.profile.perChannelActive[num] then 
				if CookyChat.db.profile.groupChannelActive[num][8] then
					SendChatMessage(reply,"OFFICER")
				end
			else
				SendChatMessage(reply,"OFFICER")
			end
		end
	end
end

-- Filler variables for the main reply functionality of the addon
local compareMessage
local trigger

-- Reply for achviement procs
function CookyChat:AchievementReply(event, message, shortSource)
	local achievementName = nil
	local reply
	
	if CookyChat.db.profile.numFilled ~= 0 and (shortSource ~= UnitName("player")) then
		for i = 1, CookyChat.db.profile.numFilled do
			if CookyChat:PercentageCheck(i) and (CookyChat.db.profile.groupTriggerType[i] == "achievementSelect") and CookyChat.db.profile.globalActive and CookyChat.db.profile.groupActive[i] and CookyChat:OnGroupWhitelist(i, shortSource) and not spamBlock then
				reply = CookyChat.db.profile.reply[i]
				
				C_Timer.After(1, 
					function()
						spamBlock = false
					end)
				
				if string.find(reply, "%%A") then
					local startIndex
					_, startIndex = string.find(message, "achievement")
					startIndex = startIndex + 2
					achievementName = string.sub(message, startIndex)
					
					reply = string.gsub(reply, "%%A", achievementName)
				end
				if string.find(reply, "%%S") then
					reply = string.gsub(reply, "%%S", shortSource)
				end
				
				spamBlock = true
				if event == "CHAT_MSG_GUILD_ACHIEVEMENT" then
					if CookyChat.db.profile.perChannelActive[i] then
						if CookyChat.db.profile.groupChannelActive[i][4] then
							C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
								SendChatMessage(reply,"GUILD")
							end)
						end
					else
						C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
							SendChatMessage(reply,"GUILD")
						end)
					end
				elseif event == "CHAT_MSG_ACHIEVEMENT" then
					if CookyChat.db.profile.perChannelActive[i] then
						if CookyChat.db.profile.groupChannelActive[i][1] then
							C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
								SendChatMessage(reply,"SAY")
							end)
						end
					else
						C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
							SendChatMessage(reply,"SAY")
						end)
					end
				end
				
			end
		end
	end
end

-- Reply for guild joining
function CookyChat:GuildJoinReply(shortSource)
	local guildName = GetGuildInfo("player")
	local reply
	
	if CookyChat.db.profile.numFilled ~= 0 and (shortSource ~= UnitName("player")) then
		for i = 1, CookyChat.db.profile.numFilled do
			if CookyChat:PercentageCheck(i) and (CookyChat.db.profile.groupTriggerType[i] == "guildJoiningSelect") and CookyChat.db.profile.groupActive[i] and CookyChat:OnGroupWhitelist(i, shortSource) then
				reply = CookyChat.db.profile.reply[i]
				
				C_Timer.After(10, function() 
					msgNumber = 0 
					spamBlock = false 
				end)
				
				msgNumber = msgNumber + 1
					
				if msgNumber == CookyChat.db.profile.spamAmount then
					spamBlock = true
				end
				
				if string.find(reply, "%%G") then
					reply = string.gsub(reply, "%%G", guildName)
				end
				if string.find(reply, "%%S") then
					reply = string.gsub(reply, "%%S", shortSource)
				end
				
				C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
					SendChatMessage(reply,"GUILD")
				end)
			end
		end
	end
end

-- Reply for guild logging on
function CookyChat:GuildLogOnReply(shortSource)
	local guildName = GetGuildInfo("player")
	local reply
	
	if CookyChat.db.profile.numFilled ~= 0 and (shortSource ~= UnitName("player")) then
		for i = 1, CookyChat.db.profile.numFilled do
			if CookyChat:PercentageCheck(i) and (CookyChat.db.profile.groupTriggerType[i] == "guildLogInSelect") and CookyChat.db.profile.groupActive[i] and CookyChat:OnGroupWhitelist(i, shortSource) and not spamBlock then
		
				reply = CookyChat.db.profile.reply[i]
				
				C_Timer.After(10, function() 
						msgNumber = 0 
						spamBlock = false 
					end)
					
				msgNumber = msgNumber + 1
					
				if msgNumber == CookyChat.db.profile.spamAmount then
					spamBlock = true
				end
				
				if string.find(reply, "%%G") then
					reply = string.gsub(reply, "%%G", guildName)
				end
				if string.find(reply, "%%S") then
					reply = string.gsub(reply, "%%S", shortSource)
				end
				
				
				C_Timer.After((CookyChat.db.profile.groupDelay[i] + 3), function()
					SendChatMessage(reply,"GUILD")
				end)
			end
		end
	end
end

-- Check for exact matching 
function CookyChat:ExactCheck(compareMessage, trigger)
	local checkTbl = {}
	
	for v in string.gmatch(compareMessage, "%S+") do
		table.insert(checkTbl, v)
	end
	
	for i = 1, #checkTbl do
		if trigger == checkTbl[i] then
			return true
		end
	end
	
	return false
end

-- Create Other Channel Table for use with replying to other channels
local otherChannelTable
local otherChannelConversion = {
	"CHAT_MSG_SAY", -- Say 1
	"CHAT_MSG_PARTY", -- Party 2
	"CHAT_MSG_RAID", -- Raid 3
	"CHAT_MSG_GUILD", -- Guild 4
	"CHAT_MSG_YELL", -- Yell 5
	"CHAT_MSG_OFFICER", -- Officer 6
	"trigger", -- Triggered Channel 7
}

function CookyChat:FillOtherChannelTable(groupNum)
	otherChannelTable = {}
	-- local testMSG = ""
	
	if CookyChat.db.profile.otherChannelActive[groupNum] then
		for i = 1, #CookyChat.db.profile.otherChannelReplyActiveGroup do
			if CookyChat.db.profile.otherChannelReplyActiveGroup[groupNum][i] then
				table.insert(otherChannelTable, otherChannelConversion[i])
				-- testMSG = testMSG.." : "..otherChannelConversion[i]
			end
		end
	end
	
	if #otherChannelTable == 0 then
		table.insert(otherChannelTable, "trigger")
	end
	
	-- if groupNum == 3 then
		-- CookyChat:Print(testMSG)
	-- end
end

-- Find the trigger is the messages
function CookyChat:FindTrigger(message, event, shortSource, fullSource, pID)
	for i = 1, CookyChat.db.profile.numFilled do
		if CookyChat.db.profile.groupActive[i] and CookyChat.db.profile.groupTriggerType[i] == "chatMessageSelect" then
			if message then
				if not CookyChat.db.profile.groupCaseSensitive[i] then 
					compareMessage = string.lower(message)
					trigger = string.lower(CookyChat.db.profile.trigger[i])
				else
					compareMessage = message
					trigger = CookyChat.db.profile.trigger[i]
				end
				
				CookyChat:FillOtherChannelTable(i)
				
				if not spamBlock and CookyChat.db.profile.exactMatch[i] and CookyChat:ExactCheck(compareMessage, trigger) then
					if event == "CHAT_MSG_BN_WHISPER" and CookyChat:PercentageCheck(i) then
						C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
							CookyChat:ChatMsgReply(event, i, _, _, pID, otherChannelTable)
						end)		
					else
						if CookyChat:OnGroupWhitelist(i, shortSource) and CookyChat:PercentageCheck(i) then
							C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
								CookyChat:ChatMsgReply(event, i, shortSource, fullSource, _, otherChannelTable)
							end)
						end
					end
					
				elseif not spamBlock and string.find(compareMessage, trigger) then
					if event == "CHAT_MSG_BN_WHISPER" and CookyChat:PercentageCheck(i) then
						C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
							CookyChat:ChatMsgReply(event, i, _, _, pID, otherChannelTable)
						end)
					else
						if CookyChat:OnGroupWhitelist(i, shortSource) and CookyChat:PercentageCheck(i) then
							C_Timer.After(CookyChat.db.profile.groupDelay[i], function()
								CookyChat:ChatMsgReply(event, i, shortSource, fullSource, _, otherChannelTable)
							end)
						end
					end
				end
			end
		end
	end		
end

-- Check if person on blacklist
function CookyChat:OnBlacklist(event, shortSource)
	if CookyChat.db.profile.blacklist ~= "" then
		local blacklistTbl = {}
		if CookyChat.db.profile.blacklist then
			for v in string.gmatch(CookyChat.db.profile.blacklist, "%S+") do
				table.insert(blacklistTbl, v)
			end
		end
	
		for i = 1, #blacklistTbl do
			if blacklistTbl[i] and shortSource then
				if string.lower(blacklistTbl[i]) == string.lower(shortSource) then
					return true
				end
			end
		end
		
		return false
	end
	
	return false
end

-- Check if person is on global whitelist
function CookyChat:OnWhitelist(event, shortSource)
	if CookyChat.db.profile.globalWhitelist ~= "" then
		local whitelistTbl = {}
		
		if CookyChat.db.profile.globalWhitelist then
			for v in string.gmatch(CookyChat.db.profile.globalWhitelist, "%S+") do
				table.insert(whitelistTbl, v)
			end
		end
		
		for i = 1, #whitelistTbl do
			if whitelistTbl[i] and shortSource then
				if string.lower(whitelistTbl[i]) == string.lower(shortSource) then
					return true
				end
			end
		end
	end
	return false
end

-- Check if person is on whitelist for individual groups
function CookyChat:OnGroupWhitelist(num, shortSource)
	if CookyChat.db.profile.whitelist[num] == "" then
		return true
	end
	
	local whitelistTbl = {}
	if CookyChat.db.profile.whitelist[num] then
		for v in string.gmatch(CookyChat.db.profile.whitelist[num], "%S+") do
			table.insert(whitelistTbl, v)
		end
	end
	
	for i = 1, #whitelistTbl do
		if whitelistTbl[i] and shortSource then
			if string.lower(whitelistTbl[i]) == string.lower(shortSource) then
				return true
			end
		end
	end
	
	return false
end

-- Main Script
CookyChatHandler:SetScript("OnEvent", function(_,event,message,fullSource,_,_,shortSource,_,_,_,_,_,_,_,pID)
	if event == "CHAT_MSG_SYSTEM" then 
		-- Joining Guild
		if message and string.find(message, "has joined the guild.") then
			shortSource = string.sub(message, 1, (string.find(message, "-") - 1))
			
			if not CookyChat:OnBlacklist(event, shortSource) and ((CookyChat:OnWhitelist(event, shortSource) and CookyChat.db.profile.semiOnOff) or CookyChat.db.profile.globalActive) then
				CookyChat:GuildJoinReply(shortSource)
			end
		end
		
		-- Guild Logging On
		if message and string.find(message, "has come online.") then
			local startIndex = string.find(message, "%[") + 1
			local endIndex = string.find(message, "%]") - 1
			shortSource = string.sub(message, startIndex, endIndex)
			
			if not CookyChat:OnBlacklist(event, shortSource) and ((CookyChat:OnWhitelist(event, shortSource) and CookyChat.db.profile.semiOnOff) or CookyChat.db.profile.globalActive) then
				CookyChat:GuildLogOnReply(shortSource)
			end
		end

	elseif not CookyChat:OnBlacklist(event, shortSource) and ((CookyChat:OnWhitelist(event, shortSource) and CookyChat.db.profile.semiOnOff) or CookyChat.db.profile.globalActive) then
		-- Achivement Earned
		if event == "CHAT_MSG_GUILD_ACHIEVEMENT" or event == "CHAT_MSG_ACHIEVEMENT" then
			CookyChat:AchievementReply(event, message, shortSource)
			
		-- Regular Chat Messages
		else
			CookyChat:FindTrigger(message, event, shortSource, fullSource, pID)
		end
	end
end)